{% extends "standingsrequests/base.html" %}
{% load static i18n %}

{% block page_content %}
    <div ng-app="standingsApp" ng-controller="GroupsListController">
        <standings-filter-controls></standings-filter-controls>

        <h2 class="page-header">
            {% trans "Groups" %} ({{groups_count|default:"-"}})
        </h2>

        {% include "standingsrequests/partials/_contacts_legend.html" %}

        <div id="div_spinner">
            {% include "standingsrequests/partials/_spinner.html" %}
        </div>

        <div id="div_results" class="container-fluid" style="visibility: hidden">
            <div class="row">
                {% include "standingsrequests/partials/_view_groups_corporations.html" %}
                {% include "standingsrequests/partials/_view_groups_alliances.html" %}
            </div>
        </div>

    </div>
{% endblock page_content %}

{% block extra_javascript %}
    <script type="text/javascript">
        {% include 'standingsrequests/partials/_angular_urls.html' %}
    </script>
    <script src="{% static 'standingsrequests/app.js' %}"></script>
    <script src="{% static 'standingsrequests/standings_filter.js' %}"></script>
    <script src="{% static 'standingsrequests/view_groups.js' %}"></script>
{% endblock extra_javascript %}
