{% extends "admin/change_list.html" %}
{% load i18n static %}

{% block object-tools-items %}
{{ block.super }}
<li>
    <a href="{% url 'standingsrequests:admin_changeset_update_now' %}" class="btn btn-high">Update now</a>
</li>
{% endblock %}
