# Generated by Django 3.2.9 on 2021-12-01 18:58

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0005_type_materials_and_sections"),
        ("standingsrequests", "0008_add_revocation_reason"),
    ]

    operations = [
        migrations.AlterField(
            model_name="corporationdetails",
            name="ceo",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="eveuniverse.eveentity",
            ),
        ),
    ]
